function coord(x,y){return {x:x,y:y};}

function withInBounds(map){
	if(!map)return map;
	map.inBounds=function(x,y){
		if(typeof x != 'number'){ // handle the arg being a coord
			y=x.y;x=x.x;
		}
		return x>=0 && y>=0 && x<this.width && y<this.height;
	}
	return map;
}

function withNeighbors(map){
	if(!map)return map;
		
	map.neighbors=function(x,y,diag){
		if(typeof x != 'number'){ // handle the arg being a coord
			diag=y;
			y=x.y;x=x.x;
		}

		var inBounds=function(x,y){
			var b=(x>=0 && y>=0 && x<map.width && y<map.height);
			return b;
		}

		diag=diag||false;
		
		var c=[];
		
		if(inBounds(x-1,y)){c.push(coord(x-1,y));}
		if(inBounds(x+1,y)){c.push(coord(x+1,y));}
		if(inBounds(x,y-1)){c.push(coord(x,y-1));}
		if(inBounds(x,y+1)){c.push(coord(x,y+1));}
		
		if(diag){
			if(inBounds(x-1,y-1)){c.push(coord(x-1,y-1));}
			if(inBounds(x+1,y-1)){c.push(coord(x+1,y-1));}
			if(inBounds(x-1,y+1)){c.push(coord(x-1,y+1));}
			if(inBounds(x+1,y+1)){c.push(coord(x+1,y+1));}
		}
		
		return c;
	}
	return map;
}


function withSpacesHaving(map){
	if(!map)return map;

	map.spacesHaving=function(pred){
		if(typeof pred == 'string'){
			var type=pred;
			pred=function(piece){return piece.type==type;}
		}
		
		var c=[];
		
		for(var y=0;y<this.height;y++){
			for(var x=0;x<this.width;x++){
				var pcs=this.pieces(x,y);
				for(var i in pcs){
					if(pred(pcs[i])){
						c.push(coord(x,y));
						break;
					}
				}
			}
		}
		return c;
	}
	
	return map;
}


function withTypeAt(map){
	if(!map)return map;
	
	map.typeAt=function(type,x,y){
		if(typeof x != 'number'){
			y=x.y;x=x.x;
		}
		if(this.inBounds(x,y)){
			var pcs=this.pieces(x,y);
			for(var i in pcs){
				if(pcs[i].type==type){
					return pcs[i];
				}
			}
		}
		return null;
	}
	
	return map;
}


function makeSelector(itemlist,selColor,selFgColor){
	itemlist.selected=null;
	var currentOrigColor=null;
	var currentFgOrigColor=null;
	var onclick=function(newSelection){
		if(itemlist.selected!=null){
			itemlist[itemlist.selected].background=currentOrigColor;
			itemlist[itemlist.selected].foreground=currentFgOrigColor;
		}
		itemlist.selected=newSelection;
		currentOrigColor=itemlist[itemlist.selected].background;
		currentFgOrigColor=itemlist[itemlist.selected].foreground;
		itemlist[itemlist.selected].background=selColor;
		if(selFgColor)
			itemlist[itemlist.selected].foreground=selFgColor;
		itemlist.repaint();
	}
	
	itemlist.selectcallback=onclick;
	return onclick;
}


function makeTypeFactory(type,details){
	details=details || {shape:'circle',color:'black'};
	type=type||null;
	
	return function(){
		var c=new Piece(details);
		c.type=type;
		return c;
	}
}


/*
delta({1,3},{0,5}) => {-1,2}
*/
function delta(a,b){
	return coord(b.x-a.x,b.y-a.y);
}

function unit(a){
	d=coord(a.x,a.y);
	
	if(d.x<0){d.x=-1;}
	else if(d.x>0){d.x=1;}

	if(d.y<0){d.y=-1;}
	else if(d.y>0){d.y=1;}

	return d;
}
/*
along({1,3},{-1,2}) => {0,5}
along(a,delta(a,b)) => b
*/
function along(a,d){
	return coord(a.x+d.x,a.y+d.y);
}

function toward(a,b){
	return along(a,unit(delta(a,b)));
}