var sidebar=createSidebar();
var world=new Map(10,10,32);
Window.sidebar=sidebar;
Window.map=world;

selected=new Piece({color:'red',shape:'outline'});

world.spaces=function(x,y){
	if((x+y)%2==0)return 'black';
	return {color:'gray',shape:'cross'};
}
///////// Sidebar stuff //////////
function createSidebar(){
	var s=new Sidebar();
	s[0]={type:'space'};
	s[1]='';
	return s;
}

function setDescription(desc){
	sidebar[1]=desc;
}

////////// Map stuff ////////////
function selectSpace(x,y,b){
	if(b===1){
		world.place(selected,x,y);
	}else{
		if(world.pieces(x,y).length==0){
			var pipe=makePipe('white');
			world.place(pipe,x,y);
			connectAdjacent(pipe);
		}
	}
	setDescription(findDescription(world,x,y));
}

world.onclick=selectSpace;
findDescription=function(w,x,y){return itself(world.spaces(x,y));};

////////// Utils ////////////
function itself(x){return x;}
function randomCoord(map,empty){
	var x=random(map.width);
	var y=random(map.height);
	if(empty)
		while(map.pieces(x,y).length>0){
			x=random(map.width);
			y=random(map.height);
		}
	return {x:x,y:y};
}
function inBounds(x,y){
	return x>=0 && x<world.width && y>=0 && y<world.width;
}
function empty(x,y,dx,dy){
	dx=dx||0;
	dy=dy||0;
	x+=dx;y+=dy;
	return inBounds(x,y) && world.pieces(x,y).length==0;
}
function filter(list,pred){ // only members of list where pred is true
	var ret=[];
	for(var i in list){
		if(pred(list[i])){
			ret.push(list[i]);
		}
	}
	return ret;
}
function rand(list){ // random member of list
	if(list.length==0){return null;}
	return list[random(list.length)];
}
function foreach(list,fn){
	for(var i in list){
		fn(list[i]);
	}
}
////////// Pieces ///////////
function makeTypeConstructor(type,shape,defaultColor){
	defaultColor=defaultColor || 'black';
	return function(color,at){
		color=color || defaultColor;
		var x=new Piece({color:color,shape:shape});
		x.type=type;
		if(at){
			world.place(x,at.x,at.y);
		}
		return x;
	}
}
makeFactory=makeTypeConstructor('factory','cross');
makeResource=makeTypeConstructor('resource','square');
makeCollector=makeTypeConstructor('collector','circle');

makePipe=function(){
	var cons=makeTypeConstructor('pipe','lines');
	return function(color,at){
		var p=cons(color,at);
		p.alone=function(){
			return !(this.details.north || this.details.south ||
				this.details.east || this.details.west);
		}
		p.lonely=function(){
			if(this.alone() && this.details.shape!=='circle'){
				this.details.shape='circle';
			}else if(!this.alone() && this.details.shape!=='lines'){
				this.details.shape='lines';
			}
		}
		return p;
	}
}();

makeCreep=function(){
	var cons=makeTypeConstructor('creep','circle');
	return function(color,at){
		var p=cons(color,at);
		p.creep=function(){
			var dirs=[{x:0,y:-1},{x:0,y:1},{x:-1,y:0},{x:1,y:0}];
			var cx=this.x,cy=this.y;
			var dir=rand(filter(dirs,function(d){
				return empty(cx,cy,d.x,d.y);
			}));
			if(dir){
				// print(cx+', '+cy+' + '+dir.x+', '+dir.y);
				this.move(cx+dir.x,cy+dir.y);
			}
		}
		return p;
	}
}();

function placeRandom(maker,color, num){
	var pcs=[];
	for(var n=0;n<num;n++)
		pcs.push(maker(color,randomCoord(world,true)));
	return pcs;
}

//placeRandom(makeResource,'red',10);
//placeRandom(makeResource,'blue',10);
placeRandom(makeResource,'green',15);

placeRandom(makeFactory,'white',1);

creeps=placeRandom(makeCreep,'red',5);
//////////// Creeps ///////////
function moveCreeps(){
	foreach(creeps,function(c){c.creep();});
}

//////////// Pipes ///////////
function pipeInDelta(dx,dy){
	return function(x,y){
		var nx=dx+x,ny=dy+y;
		if(!inBounds(nx,ny)){
			return null;
		}
		var pcs=world.pieces(nx,ny);
		for(var i in pcs){
			if(pcs[i].type==='pipe'){
				return pcs[i];
			}
		}
		return null;
	}
}

function oppositeDir(dir){
	if(dir=='north')return 'south';
	if(dir=='south')return 'north';
	if(dir=='east')return 'west';
	if(dir=='west')return 'east';
	return null;
}

function dirName(dx,dy){
	if(dx==1)return 'east';
	if(dx==-1)return 'west';
	if(dy==1)return 'south';
	if(dy==-1)return 'north';
	return null;
}

function dirDeltas(name){
	if(dir=='north')return {x:0,y:-1};
	if(dir=='south')return {x:0,y:1};
	if(dir=='east')return {x:1,y:0};
	if(dir=='west')return {x:-1,y:0};
	return {x:0,y:0};
}

function connectInDelta(dx,dy){
	var getPipe=pipeInDelta(dx,dy);
	return function(piece){
		var pipe=getPipe(piece.x,piece.y);
		if(pipe){
			var dir=dirName(dx,dy);
			piece.details[dir]=true;
			pipe.details[oppositeDir(dir)]=true;
			pipe.lonely();
			piece.repaint();
			pipe.repaint();
		}
	}
}

var connectN=connectInDelta(0,-1);
var connectS=connectInDelta(0,1);
var connectE=connectInDelta(1,0);
var connectW=connectInDelta(-1,0);

function connectAdjacent(piece){
	connectN(piece);
	connectS(piece);
	connectE(piece);
	connectW(piece);
	piece.lonely();
}

/////////////// Lines //////////////
function tempLine(x1,y1,x2,y2){
	if(x1==x2 && y1==y2){return [];}
	if(x1!=x2 && y1!=y2){return [];}
	var pcs=[];
	if(x1==x2){ // vertical line
		if(y1>y2){var t=y1;y1=y2;y2=t;}
		// for(var y=y1;)
	}
}