(defun zorch-line ()
  "Kill the line the point is on, from anywhere on that line"
  (interactive)
  (save-excursion
    (kill-new "")
    (kill-region (line-beginning-position)
                         (if (= (line-end-position) (point-max))
                                    (line-end-position)
                                    (+ 1 (line-end-position))))
    ))
 
(defun duplicate-line ()
  "Duplicate the line the point is on, moving the point to the duplicate"
  (interactive)
  (save-excursion
    (kill-new (buffer-substring (line-beginning-position) (line-end-position)))
    (end-of-line)
    (newline)
    (yank)
    (rotate-yank-pointer 1)
    )
  (next-line 1)
  )
 
(defun entag (start end tagname)
  "Surround region with HTML tag"
  (interactive "r\nsTag name: ")
  (save-excursion
    (goto-char end)
    (insert "</" tagname ">")
    (goto-char start)
    (insert "<" tagname ">")
    )
  (goto-char (+ start 1 (length tagname)))
  (if (equal "a" tagname)
      (progn
	(insert " href=\"\"")
	(forward-char -1))))

(defun copy-whole-buffer ()
  "Add a new entry to the kill ring containing the entire buffer"
  (interactive)
  (kill-new (buffer-substring (point-min) (point-max)))
  (message "Buffer %s copied to kill ring" (buffer-name)))
 
(defun roll-line-down ()
  "Move current line downward once."
  (interactive)
  (let ((col (- (point) (line-beginning-position))))
    (forward-line)
    (transpose-lines 1)
    (forward-line -1)
    (forward-char col)))
 
(defun roll-line-up ()
  "Move current line upward once."
  (interactive)
  (let ((col (- (point) (line-beginning-position))))
    (transpose-lines 1)
    (forward-line -2)
    (forward-char col)))

(global-set-key [(meta up)] 'roll-line-up)
(global-set-key [(meta down)] 'roll-line-down)
(global-set-key "\M-k" 'zorch-line)
(global-set-key "\M-d" 'duplicate-line)
(global-set-key "\C-e" 'entag)
(global-set-key "\C-a" 'copy-whole-buffer)
